jcm_root = getenv('JCMROOT'); % Linux: use: jcm_root = <JCMROOT> -> set your JCMROOT installation directory
addpath(fullfile(jcm_root, 'ThirdPartySupport', 'Matlab'));

%% shutdown a possibly running daemon
jcmwave_daemon_shutdown();

%% register a new computer resource
jcmwave_daemon_add_workstation('Hostname', 'localhost', ...
			       'Multiplicity', 1, ...
			       'NThreads', 2);

% Main script

% Define the paths
root_dir = fileparts(mfilename('fullpath'));
data_dir = fullfile(fileparts(root_dir), 'data');
project_file = fullfile(root_dir, 'jcm', 'project.jcmp');

% Main function to run the script
main(data_dir, project_file);

function main(data_dir, project_file)
    % Model parameters
    model_param_keys = struct('h', 55, 'width', 31, 'swa', 88, 'radius', 8);

    % Rest of the parameters for the JCMsuite project
    keys = struct('derivative_order', 0, 'fem_degree', 3, 'precision', 1e-3, ...
                  'n1', 1, 'n2', 1.4, 'n3', 1.967 + 4.443 * 1i, ...
                  'theta', 65, 'phi', 45, 'vacuum_wavelength', 365e-9);
              
    % Merge the two
    keys = mergeStructs(keys, model_param_keys);

    wl_count = 51;

    wavelengths = linspace(266, 800, wl_count);

    material = loadSiMaterial(data_dir);

    job_ids = [];

    for wavelength = wavelengths
        % Get a local copy of the keys for modifying and dispatching
        inner_keys = keys;
        inner_keys.vacuum_wavelength = wavelength * 1e-9;
        inner_keys.n3 = get_nk(material, wavelength);
        job_id = jcmwave_solve(project_file, inner_keys, 'temporary', 'yes');
        job_ids = [job_ids, job_id];
    end

    % Here we wait until all jobs are finished
    [results, logs] = jcmwave_daemon_wait(job_ids); % Assuming a similar function exists

    % Initialize mueller_matrices
    mueller_matrices = zeros(length(wavelengths), 4, 4);

    % Iterate over all results
    for idx = 1:length(results)
        result = results{idx};
        % First get the Mueller matrix
        M = result{3}.Mueller_ps;  % Corrected indexing

        % Extract the 4x4 matrix from the cell array
        M_matrix = M{1};
        mueller_matrices(idx, :, :) = M_matrix;
    end


    fig = figure;
    tiledlayout(4, 4, 'TileSpacing', 'Compact');
    sgtitle('Mueller matrix entries');  % Use sgtitle for the figure title
    for i = 1:4
        for j = 1:4
            nexttile;
            plot(wavelengths, squeeze(mueller_matrices(:, i, j)));
            title(['M' num2str(i) num2str(j)]);
            if i == 4
                xlabel('Wavelength (nm)');
            end
        end
    end

    % Save the plot to a file
    saveas(fig, 'mueller_matrix_plot_matlab.png');
end

function material = loadSiMaterial(data_dir)
    material_file = fullfile(data_dir, 'Aspnes.csv');
    
    % Read the material file
    n_data = readtable(material_file, 'Range', 'A2:B47');
    k_data = readtable(material_file, 'Range', 'A49:B94');
    
    % Create interpolators for material data
    material.n_interpolator = griddedInterpolant(n_data{:, 1} * 1e3, n_data{:, 2}, 'linear');
    material.k_interpolator = griddedInterpolant(k_data{:, 1} * 1e3, k_data{:, 2}, 'linear');
end

function nk = get_nk(material, wavelength)
    nk = material.n_interpolator(wavelength) + 1i * material.k_interpolator(wavelength);
end

function result = mergeStructs(struct1, struct2)
    % Merge two structures into one
    result = struct1;
    fields = fieldnames(struct2);
    for i = 1:numel(fields)
        result.(fields{i}) = struct2.(fields{i});
    end
end
