import pathlib

import jcmwave
import numpy as np
import pandas as pd
from scipy.interpolate import interp1d
from matplotlib import pyplot as plt

root_dir = pathlib.Path(__file__).parent
data_dir = root_dir.parent / "data"
project_file = root_dir / "jcm" / "project.jcmpt"
project_file_str = str(project_file.absolute())


# Register the localhost as a machine to perform computations
jcmwave.daemon.add_workstation(
    Hostname="localhost",
    #
    # Number of parallel jobs
    Multiplicity=1,
    #
    # Each parallel job runs with this number of threads. Use at least 2.
    NThreads=2,
)


# Material definitions and interpolation
class SiMaterial:
    def __init__(self):

        # Data is from refrativeindex.info
        material_file = data_dir / "Aspnes.csv"

        # Read the material file
        n_data = pd.read_csv(material_file, nrows=46)
        k_data = pd.read_csv(material_file, skiprows=48)

        # Create interpolators for material data
        self._n_interpolator = interp1d(
            n_data["wl"] * 1e3,
            n_data["n"],
            kind="linear",
        )
        self._k_interpolator = interp1d(
            k_data["wl"] * 1e3,
            k_data["k"],
            kind="linear",
        )

    def nk(self, wavelength):
        return self._n_interpolator(wavelength) + 1j * self._k_interpolator(wavelength)


def main():
    # Model parameters
    model_param_keys = dict(
        # Varying geometry
        h=55,
        width=31,
        swa=88,
        radius=8,
    )

    # Rest of the parameters for the JCMsuite project
    keys = dict(
        # Numerical parameters
        derivative_order=0,
        fem_degree=3,
        precision=1e-3,
        # Material parameters; n3 is overridden by the material file
        n1=1,
        n2=1.4,
        n3=1.967 + 4.443 * 1j,
        # Illumination
        theta=65,
        phi=45,
        vacuum_wavelength=365e-9,
    )

    # Merge the two
    keys.update(model_param_keys)

    wavelengths = np.linspace(266, 800, 51)

    material = SiMaterial()

    job_ids = list()

    for wavelength in wavelengths:

        # Get a local copy of the keys for modifying and dispatching
        inner_keys = keys.copy()
        inner_keys["vacuum_wavelength"] = wavelength * 1e-9
        inner_keys["n3"] = material.nk(wavelength)
        job_id = jcmwave.solve(project_file_str, inner_keys, temporary=True)
        job_ids.append(job_id)

    # here we wait until all jobs are finished
    results, logs = jcmwave.daemon.wait(job_ids=job_ids)

    mueller_matrices = list()

    # Iterate over all results
    for result in results:

        # First get the Mueller matrix
        M = np.zeros((4, 4))
        for i in range(4):
            for j in range(4):
                M[i, j] = result[2]["Mueller_ps{:1d}{:1d}".format(i + 1, j + 1)][0][0]
        mueller_matrices.append(M)

    # Cast to numpy and flatten
    mueller_matrices = np.asarray(mueller_matrices)
    mueller_matrices = mueller_matrices  # .flatten()

    fig, ax = plt.subplots(4, 4, figsize=(10, 10), sharex=True)

    fig.suptitle("Mueller matrix entries")

    for i in range(4):
        for j in range(4):
            # Plot data
            ax[i, j].plot(
                wavelengths,
                mueller_matrices[:, i, j],
            )

            ax[i, j].set_title(f"M{i + 1}{j + 1}")
            if i == 3:
                ax[i, j].set_xlabel("Wavelength (nm)")

    plt.tight_layout()
    # plt.show()
    plt.savefig("mueller_matrix_plot.png")


if __name__ == "__main__":
    main()
